@@ -1,13 +1,12 @@
 """ SpellChecker Module; simple, intuitive spell checker based on the post by
     Peter Norvig. See: https://norvig.com/spell-correct.html """
-from __future__ import absolute_import, division, unicode_literals
-
-import os
+import gzip
 import json
+import pkgutil
 import string
 from collections import Counter
 
-from .utils import load_file, write_file, _parse_into_words, ENSURE_UNICODE
+from .utils import _parse_into_words, ensure_unicode, load_file, write_file, deprecated
 
 
 class SpellChecker(object):
@@ -17,8 +16,8 @@ class SpellChecker(object):
 
         Args:
             language (str): The language of the dictionary to load or None \
-            for no dictionary. Supported languages are `en`, `es`, `de`, `fr` \
-            and `pt`. Defaults to `en`
+            for no dictionary. Supported languages are `en`, `es`, `de`, `fr`, \
+            `pt` and `ru`. Defaults to `en`
             local_dictionary (str): The path to a locally stored word \
             frequency dictionary; if provided, no language will be loaded
             distance (int): The edit distance to use. Defaults to 2.
@@ -50,26 +49,33 @@ class SpellChecker(object):
         if local_dictionary:
             self._word_frequency.load_dictionary(local_dictionary)
         elif language:
-            filename = "{}.json.gz".format(language.lower())
-            here = os.path.dirname(__file__)
-            full_filename = os.path.join(here, "resources", filename)
-            if not os.path.exists(full_filename):
+            filename = "resources/{}.json.gz".format(language.lower())
+            try:
+                json_open = pkgutil.get_data("spellchecker", filename)
+            except FileNotFoundError:
                 msg = (
                     "The provided dictionary language ({}) does not " "exist!"
                 ).format(language.lower())
                 raise ValueError(msg)
-            self._word_frequency.load_dictionary(full_filename)
+
+            lang_dict = json.loads(gzip.decompress(json_open).decode("utf-8"))
+            self._word_frequency.load_json(lang_dict)
 
     def __contains__(self, key):
         """ setup easier known checks """
-        key = ENSURE_UNICODE(key)
+        key = ensure_unicode(key)
         return key in self._word_frequency
 
     def __getitem__(self, key):
         """ setup easier frequency checks """
-        key = ENSURE_UNICODE(key)
+        key = ensure_unicode(key)
         return self._word_frequency[key]
 
+    def __iter__(self):
+        """ setup iter support """
+        for word in self._word_frequency.dictionary:
+            yield word
+
     @property
     def word_frequency(self):
         """ WordFrequency: An encapsulation of the word frequency `dictionary`
@@ -107,7 +113,7 @@ class SpellChecker(object):
                 text (str): The text to split into individual words
             Returns:
                 list(str): A listing of all words in the provided text """
-        text = ENSURE_UNICODE(text)
+        text = ensure_unicode(text)
         return self._tokenizer(text)
 
     def export(self, filepath, encoding="utf-8", gzipped=True):
@@ -120,9 +126,9 @@ class SpellChecker(object):
         data = json.dumps(self.word_frequency.dictionary, sort_keys=True)
         write_file(filepath, encoding, gzipped, data)
 
-    def word_probability(self, word, total_words=None):
-        """ Calculate the probability of the `word` being the desired, correct
-            word
+    def word_usage_frequency(self, word, total_words=None):
+        """ Calculate the frequency to the `word` provided as seen across the
+            entire dictionary
 
             Args:
                 word (str): The word for which the word probability is \
@@ -132,11 +138,32 @@ class SpellChecker(object):
                 frequency
             Returns:
                 float: The probability that the word is the correct word """
-        if total_words is None:
+        if not total_words:
             total_words = self._word_frequency.total_words
-        word = ENSURE_UNICODE(word)
+        word = ensure_unicode(word)
         return self._word_frequency.dictionary[word] / total_words
 
+    @deprecated("Deprecated as of version 0.6.1; use word_usage_frequency instead")
+    def word_probability(self, word, total_words=None):
+        """ Calculate the frequency to the `word` provided as seen across the
+            entire dictionary; function was a misnomar and is therefore
+            deprecated!
+
+            Args:
+                word (str): The word for which the word probability is \
+                calculated
+                total_words (int): The total number of words to use in the \
+                calculation; use the default for using the whole word \
+                frequency
+            Returns:
+                float: The probability that the word is the correct word
+            Note:
+                Deprecated as of version 0.6.1; use `word_usage_frequency` \
+                instead
+            Note:
+                Will be removed in version 0.6.3 """
+        return self.word_usage_frequency(word, total_words)
+
     def correction(self, word):
         """ The most probable correct spelling for the word
 
@@ -144,9 +171,9 @@ class SpellChecker(object):
                 word (str): The word to correct
             Returns:
                 str: The most likely candidate """
-        word = ENSURE_UNICODE(word)
+        word = ensure_unicode(word)
         candidates = list(self.candidates(word))
-        return max(sorted(candidates), key=self.word_probability)
+        return max(sorted(candidates), key=self.__getitem__)
 
     def candidates(self, word):
         """ Generate possible spelling corrections for the provided word up to
@@ -156,7 +183,7 @@ class SpellChecker(object):
                 word (str): The word for which to calculate candidate spellings
             Returns:
                 set: The set of words that are possible candidates """
-        word = ENSURE_UNICODE(word)
+        word = ensure_unicode(word)
         if self.known([word]):  # short-cut if word is correct already
             return {word}
 
@@ -184,13 +211,12 @@ class SpellChecker(object):
             Returns:
                 set: The set of those words from the input that are in the \
                 corpus """
-        words = [ENSURE_UNICODE(w) for w in words]
+        words = [ensure_unicode(w) for w in words]
         tmp = [w if self._case_sensitive else w.lower() for w in words]
         return set(
             w
             for w in tmp
-            if w in self._word_frequency.dictionary
-            and self._check_if_should_check(w)
+            if w in self._word_frequency.dictionary and self._check_if_should_check(w)
         )
 
     def unknown(self, words):
@@ -202,7 +228,7 @@ class SpellChecker(object):
             Returns:
                 set: The set of those words from the input that are not in \
                 the corpus """
-        words = [ENSURE_UNICODE(w) for w in words]
+        words = [ensure_unicode(w) for w in words]
         tmp = [
             w if self._case_sensitive else w.lower()
             for w in words
@@ -219,7 +245,11 @@ class SpellChecker(object):
             Returns:
                 set: The set of strings that are edit distance one from the \
                 provided word """
-        word = ENSURE_UNICODE(word).lower() if not self._case_sensitive else ENSURE_UNICODE(word)
+        word = (
+            ensure_unicode(word).lower()
+            if not self._case_sensitive
+            else ensure_unicode(word)
+        )
         if self._check_if_should_check(word) is False:
             return {word}
         letters = self._word_frequency.letters
@@ -239,7 +269,11 @@ class SpellChecker(object):
             Returns:
                 set: The set of strings that are edit distance two from the \
                 provided word """
-        word = ENSURE_UNICODE(word).lower() if not self._case_sensitive else ENSURE_UNICODE(word)
+        word = (
+            ensure_unicode(word).lower()
+            if not self._case_sensitive
+            else ensure_unicode(word)
+        )
         return [
             e2 for e1 in self.edit_distance_1(word) for e2 in self.edit_distance_1(e1)
         ]
@@ -253,7 +287,7 @@ class SpellChecker(object):
             Returns:
                 set: The set of strings that are edit distance two from the \
                 provided words """
-        words = [ENSURE_UNICODE(w) for w in words]
+        words = [ensure_unicode(w) for w in words]
         tmp = [
             w if self._case_sensitive else w.lower()
             for w in words
@@ -264,7 +298,9 @@ class SpellChecker(object):
     def _check_if_should_check(self, word):
         if len(word) == 1 and word in string.punctuation:
             return False
-        if len(word) > self._word_frequency.longest_word_length + 3:  # magic number to allow removal of up to 2 letters.
+        if (
+            len(word) > self._word_frequency.longest_word_length + 3
+        ):  # magic number to allow removal of up to 2 letters.
             return False
         try:  # check if it is a number (int, float, etc)
             float(word)
@@ -286,7 +322,7 @@ class WordFrequency(object):
         "_letters",
         "_tokenizer",
         "_case_sensitive",
-        "_longest_word_length"
+        "_longest_word_length",
     ]
 
     def __init__(self, tokenizer=None, case_sensitive=False):
@@ -303,16 +339,21 @@ class WordFrequency(object):
 
     def __contains__(self, key):
         """ turn on contains """
-        key = ENSURE_UNICODE(key)
+        key = ensure_unicode(key)
         key = key if self._case_sensitive else key.lower()
         return key in self._dictionary
 
     def __getitem__(self, key):
         """ turn on getitem """
-        key = ENSURE_UNICODE(key)
+        key = ensure_unicode(key)
         key = key if self._case_sensitive else key.lower()
         return self._dictionary[key]
 
+    def __iter__(self):
+        """ turn on iter support """
+        for word in self._dictionary:
+            yield word
+
     def pop(self, key, default=None):
         """ Remove the key and return the associated value or default if not
             found
@@ -320,7 +361,7 @@ class WordFrequency(object):
             Args:
                 key (str): The key to remove
                 default (obj): The value to return if key is not present """
-        key = ENSURE_UNICODE(key)
+        key = ensure_unicode(key)
         key = key if self._case_sensitive else key.lower()
         return self._dictionary.pop(key, default)
 
@@ -374,8 +415,9 @@ class WordFrequency(object):
             Yields:
                 str: The next `word` in the tokenized string
             Note:
-                This is the same as the `spellchecker.split_words()` """
-        text = ENSURE_UNICODE(text)
+                This is the same as the `spellchecker.split_words()` unless \
+                a tokenizer function was provided. """
+        text = ensure_unicode(text)
         for word in self._tokenizer(text):
             yield word if self._case_sensitive else word.lower()
 
@@ -422,6 +464,14 @@ class WordFrequency(object):
             self._dictionary.update(json.loads(data))
             self._update_dictionary()
 
+    def load_json(self, data):
+        """ Load in a pre-built word frequency list
+
+            Args:
+                data (dict): The dictionary to be loaded """
+        self._dictionary.update(data)
+        self._update_dictionary()
+
     def load_text_file(self, filename, encoding="utf-8", tokenizer=None):
         """ Load in a text file from which to generate a word frequency list
 
@@ -440,7 +490,7 @@ class WordFrequency(object):
                 text (str): The text to be loaded
                 tokenizer (function): The function to use to tokenize a string
         """
-        text = ENSURE_UNICODE(text)
+        text = ensure_unicode(text)
         if tokenizer:
             words = [x if self._case_sensitive else x.lower() for x in tokenizer(text)]
         else:
@@ -454,7 +504,7 @@ class WordFrequency(object):
 
             Args:
                 words (list): The list of words to be loaded """
-        words = [ENSURE_UNICODE(w) for w in words]
+        words = [ensure_unicode(w) for w in words]
         self._dictionary.update(
             [word if self._case_sensitive else word.lower() for word in words]
         )
@@ -465,7 +515,7 @@ class WordFrequency(object):
 
             Args:
                 word (str): The word to add """
-        word = ENSURE_UNICODE(word)
+        word = ensure_unicode(word)
         self.load_words([word])
 
     def remove_words(self, words):
@@ -473,7 +523,7 @@ class WordFrequency(object):
 
             Args:
                 words (list): The list of words to remove """
-        words = [ENSURE_UNICODE(w) for w in words]
+        words = [ensure_unicode(w) for w in words]
         for word in words:
             self._dictionary.pop(word if self._case_sensitive else word.lower())
         self._update_dictionary()
@@ -483,7 +533,7 @@ class WordFrequency(object):
 
             Args:
                 word (str): The word to remove """
-        word = ENSURE_UNICODE(word)
+        word = ensure_unicode(word)
         self._dictionary.pop(word if self._case_sensitive else word.lower())
         self._update_dictionary()
 
